/*
 *  ObjReader.cpp
 *  FinalImage
 *
 *  Created by Zach Gildersleeve on 4/12/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "ObjReader.h"
using namespace std;

static void error(char* message) {
	cerr << message;
}

void ObjReader::processFace(int &vert, int &normal, char *name)
{
	string s(name);
	int index1 = s.find_first_of("/");
	int index2 = s.find_last_of("/");
	vert = atoi(s.substr(0, index1).c_str()) - 1;
	normal = atoi(s.substr(index2+1).c_str()) - 1;
}


void ObjReader::readObj(char *filename, Material *material, Group *group)
{
	double  e = 1e-6;

	double minx = DBL_MAX;
	double miny = DBL_MAX;
	double minz = DBL_MAX;
	double maxx = -DBL_MAX;
	double maxy = -DBL_MAX;
	double maxz = -DBL_MAX;

	vector<Point> points;
	vector<Vector> normals;
	ifstream in(filename);
	if(!in)
		error("Error reading file: file does not exist");
	printf("\tObject file opened\n");
	char str[32];
	bool firstFace = true;
	int faces = 0;
	int vertCounter = 0;
	int normCounter = 0;
	
	while(in) {
		in >> str;
		if(!strcmp(str,"v")) { //vertex
			double x,y,z;
			in>>x>>y>>z;
			points.push_back(Point(x,y,z));
			vertCounter++;
			if(x < minx)
				minx = x;
			if(y < miny)
				miny = y;
			if(z < minz)
				minz = z;
			if(x > maxx)
				maxx = x;
			if(y > maxy)
				maxy = y;
			if(z > maxz)
				maxz = z;
			printf("\tReading vertex data %d\n", vertCounter);
		}
		if(!strcmp(str,"vn")) { //normal
			double x,y,z;
			in>>x>>y>>z;
			normals.push_back(Vector(x,y,z));
			normCounter++;
			printf("\tReading normal data %d\n", normCounter);

		}
		if(!strcmp(str,"f")) { //face
			if(firstFace) {
				//group->bbox.build(Point(minx - e, miny - e, minz - e));
				//group->bbox.build(Point(maxx + e, maxy + e, maxz + e));
				//group->preprocess();
			}
			char line[256];
			char s[32];
			in.getline(line, 256);
			stringstream stream(line);
			stream>>s;
			int p1, p2, p3, n1, n2, n3;
			processFace(p1, n1, s);
			stream>>s;
			processFace(p2, n2, s);
			stream>>s;
			processFace(p3, n3, s);
			
			Triangle* tri = new Triangle(material, points[p1], points[p2], points[p3]);//,
										 //normals[n1], normals[n2], normals[n3]);
			group->addObject(tri);
			faces++;
				printf("\tAdded Triangle %d at %d,%d,%d\n", faces, p1,p2,p3);
			
			//if there is another Vector in the face
			int prev = p3; //previous vertex
			int prevNormal = n3; //previous vertex
			stream>>s;
			while(s[0] != '0'){
				int p4, n4;
				processFace(p4, n4, s);
				Triangle* tri = new Triangle(material, points[p1], points[prev], points[p4]);//,
											 //normals[n1], normals[prevNormal], normals[n4]);
				faces++;
				printf("\tAdded Triangle %d at %d,%d,%d\n", faces, p1,prev,p4);
				group->addObject(tri);
				prev = p4;
				prevNormal = n4;
				stream>>s;
			}
		}
	}
//	group->postprocess();
	cerr << " min (" <<minx<<", "<<miny<<", "<<minz<<")\n";
	cerr << " max (" <<maxx<<", "<<maxy<<", "<<maxz<<")\n";
	cerr<<"\t"<<points.size()<<" points read\n";
	cerr<<"\t"<<normals.size()<<" normals read\n";
	cerr<<"\t"<<faces<<" triangles read\n";
}
