/*
 *  PhongBumpMaterial.cpp
 *  FinalImage
 *
 *  Created by Zach Gildersleeve on 4/18/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "PhongBumpMaterial.h"
#include "PerlinNoise.h"
#include <math.h>

PhongBumpMaterial::PhongBumpMaterial(const Color &newColor, double newKdiffuse, double newKambient,
							 const Color &newSpecColor, int newPhongTerm, Material *newBumpMap, double newDepth)
: color(newColor), Kdiffuse(newKdiffuse), Kambient(newKambient),
specColor(newSpecColor), phongTerm(newPhongTerm), bumpMap(newBumpMap), bumpDepth(newDepth) {
	
}

void PhongBumpMaterial::shade(Color &result, const RenderContext &context, const Ray &ray, const HitRecord &hit, int depth, const Color &attenuation) const {
	
	// P = O + tV
	Point hitPosition = ray.getOrigin() + (ray.getDirection() * hit.getT());
	
	// Call primitive to get normal - should already be normalized
	Vector normal;
	hit.getPrimitive()->normal(normal, hitPosition, hit, context, ray);
	
	//costheta = N dot V
	double cosTheta = dot(normal, ray.getDirection());
	
	if (cosTheta > 0)
		normal = -normal;
	
	Color bumpMapColor;
	bumpMap->shade(bumpMapColor, context, ray, hit, depth, attenuation);
	double bumpScale = (bumpMapColor.r + bumpMapColor.g + bumpMapColor.b) / 3.0;
	
	normal = Vector(normal.x + (bumpScale * bumpDepth), normal.y + (bumpScale * bumpDepth), normal.z + (bumpScale * bumpDepth));
	normal.normalize();
	// color light = scene.ambient * Ka
	Scene *myScene = context.scene;
	Color light = myScene->getAmbient() * Kambient;
	Color blueAmbient = Color(0.1, 0.1, 0.3);
	Vector up = Vector(0, 1, 0);
	double ambientScale = dot(up, ray.getDirection());
	ambientScale = (ambientScale + 1.0) / 2.0;
	light = light * ambientScale + blueAmbient * (1.0 - ambientScale);
	//printf("%f\n", ambientScale);
	
	
	const Object *world = myScene->object;
	
	Color currentSpecColor(0, 0, 0);
	
	// loop through all the lights
	for (std::vector<Light*>::iterator iterator = myScene->lightList.begin();
		 iterator < myScene->lightList.end(); iterator++)
	{
		// get C.L (light color) and L (vector to light, normalized in light class)
		Color currentLightColor;
		double distance;
		Vector Lnormal;
		distance =(**iterator).getLight(currentLightColor, Lnormal, context, hitPosition);
		
		// cosphi = N dot Ln
		double cosPhi = dot(normal, Lnormal);
		
		//double marbleWeight = sin(cosPhi) * ((rand() % 100) - 50) * turbulenceAbs(2, hitPosition, 10, 0);
		//printf("%f\n", marbleWeight);
		//normal *= marbleWeight;
		
		
		if (cosPhi > 0.0)
		{
			HitRecord shadowHit(distance);
			
			Ray shadowRay(hitPosition, Lnormal);
			world->intersect(shadowHit, context, shadowRay);
			
			if (!shadowHit.getPrimitive()) {
				
				light += currentLightColor * Kdiffuse * cosPhi;
				
				Vector H = Lnormal - ray.getDirection();
				H.normalize();
				
				double cosAlpha = dot(H, normal);
				if (cosAlpha > 0)
				{
					double p = pow(cosAlpha, phongTerm);
					currentSpecColor += currentLightColor * p;
				}
			}
			
		}
	}
	int r1 = (rand() % 100);
	if (r1 < 98)
		result = light * color + currentSpecColor * specColor;
	else
		result = (light * color + currentSpecColor * specColor) * 0.5;
}