/*
 *  ThinLensCamera.cpp
 *  Assignment10
 *
 *  Created by Zach Gildersleeve on 4/22/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "ThinLensCamera.h"

ThinLensCamera::ThinLensCamera(const Point &newPosition, const Point &newLookat,
			   const Vector &newUp, double newfov,
			   double newAperture, double newFocalDistance) :
				position(newPosition), lookat(newLookat),
				up(newUp), fov(newfov), aperture(newAperture), focalDistance(newFocalDistance)
{
					
}


void ThinLensCamera::preprocess(double aspectRatio) {
	normalDirection = lookat - position;
	normalDirection.normalize();
	
	u = cross(normalDirection, up);
	v = cross(u, normalDirection);
	
	double uLength = tan(fov * 3.14159 / 360.0);
	u.normalize();
	u *= uLength;
	
	double vLength = uLength / aspectRatio;
	v.normalize();
	v *= vLength;
	
	lensRadius = 5.0 / aperture;
}

void ThinLensCamera::makeRay(Ray &ray, const RenderContext &context, double x, double y) const {
	Vector myRayDirection = normalDirection + (u * x) + (v * y);
	myRayDirection.normalize();
	
	double tempX;
	double tempY;
	Vector tempU = u;
	Vector tempV = v;
	
	discRandom(tempX, tempY);
	
	Vector lensU = tempU * (tempX * lensRadius);
	Vector lensV = tempV * (tempY * lensRadius);
	
	myRayDirection = (myRayDirection * focalDistance) - (lensU + lensV);
			
	myRayDirection.normalize();
	
	ray = Ray(position + lensU + lensV, myRayDirection);
}