#include "Scene.h"
#include "ConstantBackground.h"
#include "GradientBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include "Disc.h"
#include "Ring.h"
#include "Triangle.h"
#include "Box.h"
#include "Cylinder.h"
#include "HeightField.h"
#include "PhongMaterial.h"
#include "MetalMaterial.h"
#include "DielectricMaterial.h"
#include "GriddedGroup.h"
#include "PhongMarbleMaterial.h"
#include "SmoothTriangle.h"
#include "WoodMaterial.h"
#include "TileMaterial.h"
#include "Filter.h"
#include "Sampler.h"
#include "PhongImageMaterial.h"
#include "SpherePolar.h"
#include "ObjReader.h"
#include "AreaLight.h"
#include "Instance.h"
#include "PrimitiveInstance.h"
#include "PhongBumpMaterial.h"




#include <fstream>
#include <iostream>
#include <vector>
using std::cerr;
using std::ifstream;
using std::vector;

using namespace std;

static void error(char* message) {
	cerr << message;
}

void processFace(int &vert, int &normal, char *name)
{
	string s(name);
	int index1 = s.find_first_of("/");
	int index2 = s.find_last_of("/");
	vert = atoi(s.substr(0, index1).c_str()) - 1;
	normal = atoi(s.substr(index2+1).c_str()) - 1;
}


void readObj(char *filename, Material *material, GriddedGroup *group)
{
//	double  e = 1e-6;
	
	double minx = DBL_MAX;
	double miny = DBL_MAX;
	double minz = DBL_MAX;
	double maxx = -DBL_MAX;
	double maxy = -DBL_MAX;
	double maxz = -DBL_MAX;
	
	vector<Point> points;
	vector<Vector> normals;
	ifstream in(filename);
	if(!in)
		error("Error reading file: file does not exist");
	printf("\tObject file opened\n");
	char str[32];
	bool firstFace = true;
	int faces = 0;
	int vertCounter = 0;
	int normCounter = 0;
	
	while(in) {
		in >> str;
		if(!strcmp(str,"v")) { //vertex
			double x,y,z;
			in>>x>>y>>z;
			points.push_back(Point(x,y,z));
			vertCounter++;
			if(x < minx)
				minx = x;
			if(y < miny)
				miny = y;
			if(z < minz)
				minz = z;
			if(x > maxx)
				maxx = x;
			if(y > maxy)
				maxy = y;
			if(z > maxz)
				maxz = z;
			//printf("\tReading vertex data %d\n", vertCounter);
		}
		if(!strcmp(str,"vn")) { //normal
			double x,y,z;
			in>>x>>y>>z;
			normals.push_back(Vector(x,y,z));
			normCounter++;
			//printf("\tReading normal data %d\n", normCounter);
			
		}
		if(!strcmp(str,"f")) { //face
			if(firstFace) {
				//group->bbox.build(Point(minx - e, miny - e, minz - e));
				//group->bbox.build(Point(maxx + e, maxy + e, maxz + e));
				//group->preprocess();
			}
			char line[256];
			char s[32];
			char sCopy[5];
			for(int i = 0; i < 5; i++)
				sCopy[i] = 0;
			for(int i = 0; i< 32; i++)
				s[i] = 0;
			in.getline(line, 256);
			//printf("line = %s\n", line);
			stringstream stream(line);
			stream>>s; //printf("%s\n", s);
			int p1, p2, p3, n1, n2, n3;
			processFace(p1, n1, s);
			stream>>s; //printf("%s\n", s);
			processFace(p2, n2, s);
			stream>>s;// printf("%s\n", s);
			processFace(p3, n3, s);
			
				if (p1 > 0 && p2 > 0 && p3 > 0)
				{	
					SmoothTriangle* tri = new SmoothTriangle(material, points[p1], points[p2], points[p3], normals[n1], normals[n2], normals[n3]);//,
															   
					group->addObject(tri);
					faces++;
					//printf("\tAdded Triangle %d at %d,%d,%d\n", faces, 1+p1,1+p2,1+p3);
				}
				//if there is another Vector in the face
				int prev = p3; //previous vertex
				int prevNormal = n3; //previous vertex
				for(int i = 0; i< 32; i++)
					s[i] = 0;
				stream>>s; //printf("%s\n", s);
				
				while(s[0] != '0'){
				//while(!iscntrl(s[5])){
					int p4, n4;
					processFace(p4, n4, s);
																								 
						
					if (p1 > 0 && p4 > 0 && prev > 0)
						{
							SmoothTriangle* tri = new SmoothTriangle(material, points[p1], points[prev], points[p4], normals[n1], normals[prevNormal], normals[n4]);

							faces++;
							//printf("\tAdded Triangle %d at %d,%d,%d\n", faces,1+ p1,1+prev,1+p4);
							group->addObject(tri);
						}

						
						prev = p4;
						prevNormal = n4;
						if (iscntrl(s[5]) || s[5] ==  0)
							break;
						for(int i = 0; i< 32; i++)
							s[i] = 0;
						stream>>s;// printf("s = %s\nsCopy = %s", s, sCopy);
			}
		}
	}
	//	group->postprocess();
	cerr << " min (" <<minx<<", "<<miny<<", "<<minz<<")\n";
	cerr << " max (" <<maxx<<", "<<maxy<<", "<<maxz<<")\n";
	cerr<<"\t"<<points.size()<<" points read\n";
	cerr<<"\t"<<normals.size()<<" normals read\n";
	cerr<<"\t"<<faces<<" triangles read\n";
}


Scene* make_scene()
{
//	Material* yellow = new PhongMaterial(Color(0.9, 0.9, 0), 0.6, 0.4, Color(1,1,.8), 30);
//	Material* green = new PhongMaterial(Color(30.0/255.0, 97.0/255.0, 3.0/255.5), 0.3, 0.3, Color(140.0/255.0, 239.0/255.0, 101.0/255.5), 10);
	Material* cream = new PhongMaterial(Color(238.0/255.0, 191.0/255.0, 117.0/255.5), 1.0, 1.0, Color(214.0/255.0, 235.0/255.0, 134.0/255.5), 40);

	Material* tableCloth = new PhongMarbleMaterial(Color(86.0/255.0, 77.0/255.0, 23.0/255.0), // Color 1
												   Color(0.5, 0.5, 0.1), // Color 2
												   2, // scale
												   10, // octaves
												   20.0, // turbulence scale
												   1.0, // frequency scale
												   2.0, // lacunarity
												   0.6, // gain
												   0.9, 0.9, Color(86.0/255.0, 77.0/255.0, 23.0/255.0), 120);

	Material* lemonBump = new PhongMarbleMaterial(Color(1, 1, 1), // Color 1
												   Color(0.0, 0.0, 0.0), // Color 2
												   2, // scale
												   10, // octaves
												   20.0, // turbulence scale
												   1.0, // frequency scale
												   2.0, // lacunarity
												   0.6, // gain
												   0.9, 0.9, Color(86.0/255.0, 77.0/255.0, 23.0/255.0), 120);
												   
	Material* plantBump = new PhongMarbleMaterial(Color(1, 1, 1), // Color 1
												  Color(0.0, 0.0, 0.0), // Color 2
												  5, // scale
												  2, // octaves
												  20.0, // turbulence scale
												  0.0, // frequency scale
												  10.0, // lacunarity
												  0.6, // gain
												  0.9, 0.9, Color(86.0/255.0, 77.0/255.0, 23.0/255.0), 120);
	
	

	Material* lemonMtl = new PhongBumpMaterial(Color(0.9, 0.9, 0), 0.6, 0.4, Color(1.0,1.0,1.0), 200, lemonBump, 4.0);
	Material* rindMtl = new PhongBumpMaterial(Color(0.9, 0.9, 0.7), 0.9, 0.9, Color(1.0, 1.0, 1.0), 200, lemonBump, 2.0);
	Material* pulpMtl = new PhongBumpMaterial(Color(0.9, 0.8, 0.4), 0.8, 0.4, Color(1.0,1.0,1.0), 200, lemonBump, 2.0);
	Material* purple = new PhongBumpMaterial(Color(140.0/255.0, 100.0/255.0, 124.0/255.0), 0.4, 0.4, Color(197.0/255.0, 209.0/255.0, 222.0/255.5), 30, plantBump, 2.0);
	Material* breadMtl = new PhongBumpMaterial(Color(252.0/255.0, 155.0/255.0, 58.0/255.0), 0.8, 0.8, Color(190.0/255.0, 770.0/255.0, 9.0/255.0), 50, lemonBump, 2.0);


	Material* plantMtl = new PhongBumpMaterial(Color(30.0/255.0, 97.0/255.0, 3.0/255.5), 0.4, 0.5, Color(98.0/255.0, 135.0/255.0, 28.0/255.5), 50, plantBump, 1.0);

	Material* clear = new DielectricMaterial(1.5, 100, Color(1.0, 1.0, 1.0));
	Material* greenTransparent = new DielectricMaterial(1.5, 100, Color(39.0/255.0, 95.0/255.0, 11.0/255.0));
//	Material* wineTransparent = new DielectricMaterial(1.5, 30, Color(240.0/255.0, 241.0/255.0, 41.0/255.0));

	Material * wood = new WoodMaterial();	
	Material* metalMatl = new MetalMaterial(Color(0.5, 0.5, 0.5), 200, Color(0.514, 0.365, 0.275));
	Material* bronzeMatl = new MetalMaterial(Color(0.514, 0.365, 0.275), 50, Color(1.0, .969, .745));
	Material* goldMatl = new MetalMaterial(Color(242.0/255.0, 155.0/255.0, 50.0/255.0), 20, Color(1.0, 1.0, .392));
	
	Material* marbleBlue = new PhongMarbleMaterial(Color(0.0, 0.0, 0.0), // Color 1
												   Color(1.0, 1.0, 1.0), // Color 2
												   20, // scale
												   8, // octaves
												   5.0, // turbulence scale
												   1.0, // frequency scale
												   20.0, // lacunarity
												   0.6, // gain
												   0.9, 0.9, Color(1,1,1), 120);

	Scene* scene = new Scene();
	scene->setBackground(new GradientBackground(Color(.4, .4, .4), Color(106.0/255.0, 90.0/255.0, 58.0/255.0), Vector(0, 1, 0)));
//	scene->setBackground(new ConstantBackground(Color(0.5, 0.5, 0.5)));
	Group* world = new Group();

	GriddedGroup* goblet = new GriddedGroup(10);
	world->addObject(goblet);
	
	GriddedGroup* pedestal = new GriddedGroup(10);
	world->addObject(pedestal);
	
	GriddedGroup* cloth = new GriddedGroup(10);
	world->addObject(cloth);
	
	GriddedGroup* plate1 = new GriddedGroup(10);
	world->addObject(plate1);
	GriddedGroup* plate3 = new GriddedGroup(10);
	world->addObject(plate3);
	
	GriddedGroup* lemon = new GriddedGroup(10);
	world->addObject(lemon);
	
	GriddedGroup* lemon2 = new GriddedGroup(10);
	world->addObject(lemon2);
	
	GriddedGroup* vine = new GriddedGroup(10);
	world->addObject(vine);
	
	GriddedGroup* vine2 = new GriddedGroup(10);
	world->addObject(vine2);
	
	GriddedGroup* lamp = new GriddedGroup(10);
	world->addObject(lamp);
	
	GriddedGroup* grapeGroup = new GriddedGroup(10);
	world->addObject(grapeGroup);
	
	GriddedGroup* bread = new GriddedGroup(10);
	world->addObject(bread);
	
	world->addObject(new Box(wood, Point(-20.0, -1.0, -10.0), Point(7.0, 0.0, 10.0)));	
	readObj("./cloth.obj", tableCloth, cloth);
	readObj("./goblet.obj", greenTransparent, goblet);
		
	readObj("./plate1.obj", metalMatl, plate1);
	world->addObject(new Instance(plate1, 7.973, 0, 0, 0, 0, 0, 1, 1, 1));
	readObj("./plate3.obj", metalMatl, plate3);

	readObj("./lemonWhole.obj", lemonMtl, lemon);
	readObj("./lemonHalf.obj", lemonMtl, lemon);
	readObj("./lemonRind.obj", rindMtl, lemon);
	readObj("./lemonSliceRoundPulp.obj", pulpMtl, lemon2);

	readObj("./pedestal.obj", goldMatl, pedestal);
	
	readObj("./vine.obj", plantMtl, vine);
	world->addObject(new Instance(vine, -0.904, 3.294, 0, 180, 0, 0, 1, 1, 1));
	world->addObject(new Instance(vine, 0, 0, 0, 0, 45, 0, 1, 1, 1));		
	world->addObject(new Instance(vine, -15, -1.5, 0, 0, 0, -90, 1, 1, 1));
	
	readObj("./vine2.obj", plantMtl, vine2);	

	readObj("./lampMiddle.obj", marbleBlue, lamp);
	readObj("./lampBase.obj", goldMatl, lamp);
	readObj("./lampTop.obj", cream, lamp);

	readObj("./bread.obj", breadMtl, bread);

	world->addObject(new Sphere(clear, Point(-10, 1.5, 5), 1.5));
	
	Point armiCenter = Point(-10, 6, -8);
	double scale = 2.0;
	world->addObject(new Ring(bronzeMatl, armiCenter, Vector(0, 0, 1), 2 * scale, 2.3 * scale));
	world->addObject(new Ring(bronzeMatl, armiCenter, Vector(0, 1, 0), 2 * scale, 2.5 * scale));
	world->addObject(new Cylinder(bronzeMatl, armiCenter, 2.5 * scale, 0.2 * scale));
	
	world->addObject(new Ring(bronzeMatl, armiCenter, Vector(0, 0.2, 1), 1.8 * scale, 1.95 * scale));
	world->addObject(new Ring(bronzeMatl, armiCenter, Vector(-.5, 1.0, .2), 1.8 * scale, 1.95 * scale));
	Primitive *cly = new Cylinder(bronzeMatl, Point(0, 0, 0), 1.95 * scale, 0.1 * scale);
		world->addObject(new Instance(cly, armiCenter.x, armiCenter.y, armiCenter.z, 0.2, 0, 44.49, 1, 1, 1));	
		world->addObject(new Instance(cly, armiCenter.x, armiCenter.y, armiCenter.z, 0.2, 0, 30.4, 1, 1, 1));	
	Primitive *cly2 = new Cylinder(bronzeMatl, Point(0, 0, 0), 1.6 * scale, 0.1 * scale);
		world->addObject(new Instance(cly2, armiCenter.x - .8, armiCenter.y - .4, armiCenter.z, 0.2, 0, 30.4, 1, 1, 1));
		world->addObject(new Instance(cly2, armiCenter.x + .8, armiCenter.y + .4, armiCenter.z, 0.2, 0, 30.4, 1, 1, 1));
	Primitive *cly3 = new Cylinder(bronzeMatl, Point(0, 0, 0), 1.0 * scale, 0.1 * scale);
		world->addObject(new Instance(cly3, armiCenter.x + 1.6, armiCenter.y + .8, armiCenter.z, 0.2, 0, 30.4, 1, 1, 1));
		world->addObject(new Instance(cly3, armiCenter.x - 1.6, armiCenter.y - .8, armiCenter.z, 0.2, 0, 30.4, 1, 1, 1));
	Primitive *cly4 = new Cylinder(bronzeMatl, Point(0, -2.4, 0), 0.1 * scale, 4.8 * scale);
		world->addObject(new Instance(cly4, armiCenter.x, armiCenter.y, armiCenter.z, 0.2, 0, 30.4, 1, 1, 1));
	world->addObject(new Sphere(bronzeMatl, armiCenter, 0.6 * scale));
	world->addObject(new Ring(bronzeMatl, armiCenter, Vector(1, 0.0, 1), 2.0 * scale, 2.2 * scale));
	world->addObject(new Ring(bronzeMatl, armiCenter, Vector(-1, 0.0, 1), 2.0 * scale, 2.2 * scale));

	grapeGroup->addObject(new Sphere(purple, Point(6.1, 1.1, 9.5), 0.2));
				
				for (int h = 0; h < 7; h++)
				{
					int reducer = 3;
					float taper = h * 0.35 + ((rand() % 10) * 0.02);
					for (int i = 0; i < 4 - h; i++)
					{
						float yStep = i * 0.35;
						for (int j = 0; j < 6 - h + reducer; j++)
						{
							double r1 = (rand() % 10) * 0.02;
							double radius = (rand() % 40) * 0.002;
							float xzStep = j * .35;
							grapeGroup->addObject(new Sphere(purple, Point(6.1 + -xzStep + r1 - taper, 1.1 + yStep + r1, 9.5 + -xzStep + r1 + taper), .2 + radius));
							//world->addObject(new Instance(grape, -xzStep + r1 - taper, yStep + r1, -xzStep + r1 + taper, 0, 0, 0, 1, 1, 1));
						}
						reducer--;
						if (reducer < 0)
							reducer = 0;
					}
				}
								
				for (int h = 0; h < 4; h++)
				{
					int reducer = 3;
					float taper = h * 0.35 + ((rand() % 10) * 0.02);
					for (int i = 0; i < 3 - h; i++)
					{
						float yStep = i * 0.35;
						for (int j = 0; j < 3 - h + reducer; j++)
						{
							double r1 = (rand() % 10) * 0.02;
							double radius = (rand() % 40) * 0.002;
							float xzStep = j * .35;
							grapeGroup->addObject(new Sphere(purple, Point(4.3 + -xzStep + r1 - taper, 1.3 + yStep + r1, 9.9 + -xzStep + r1 + taper), 0.2 + radius));
						}
						reducer--;
						if (reducer < 0)
							reducer = 0;
					}
				}
								
				for (int h = 0; h < 4; h++)
				{
					int reducer = 3;
					float taper = h * 0.35 + ((rand() % 10) * 0.02);
					for (int i = 0; i < 3 - h; i++)
					{
						float yStep = i * 0.35;
						for (int j = 0; j < 3 - h + reducer; j++)
						{
							double r1 = (rand() % 10) * 0.02;
							double radius = (rand() % 40) * 0.002;
							float xzStep = j * .35;
							grapeGroup->addObject(new Sphere(purple, Point(3.3 + -xzStep + r1 - taper, 0.8 + yStep + r1, 10.4 + -xzStep + r1 + taper), 0.2 + radius));
						}
						reducer--;
						if (reducer < 0)
							reducer = 0;
					}
				}
								
				for (int h = 0; h < 2; h++)
				{
					int reducer = 3;
					float taper = h * 0.35 + ((rand() % 10) * 0.02);
					for (int i = 0; i < 2 - h; i++)
					{
						float yStep = i * 0.35;
						for (int j = 0; j < 2 - h + reducer; j++)
						{
							double r1 = (rand() % 10) * 0.02;
							double radius = (rand() % 40) * 0.002;		
							float xzStep = j * .35;
							grapeGroup->addObject(new Sphere(purple, Point(5.7 + -xzStep + r1 - taper, 1.9 + yStep + r1, 9.5 + -xzStep + r1 + taper), 0.2 + radius));
						}
						reducer--;
						if (reducer < 0)
							reducer = 0;
					}
				}

	scene->setObject(world);
	
	int subDivisions = 4;
	int support = 2;
	scene->setSampler(new JitteredSampler(subDivisions));
	scene->setFilter(new TriangleFilter(support));
	
	
	scene->setAmbient(Color(.2, .2, .25));
	//scene->addLight(new PointLight(Point(20, 70, 100), Color(.9,.9,.9)));
	scene->addLight(new AreaLight(Point(20, 70, 100), Point(25, 70, 100), Point(20, 77, 100), Color(.9,.9,.9)));	
	scene->setCamera(new PinholeCamera(Point(-1,10,100),
									   Point(-1.0, 2.0, 0.0),
									   Vector(0, 1, 0),
									   12));
	scene->setMaxRayDepth(25);
	scene->setMinAttenuation(.01);
	
	return scene;
}

