#include "Scene.h"
#include "ConstantBackground.h"
#include "GradientBackground.h"
#include "Group.h"
#include "LambertMaterial.h"
#include "PointLight.h"
#include "DirectionalLight.h"
#include "PinholeCamera.h"
#include "OrthographicCamera.h"
#include "Plane.h"
#include "Disc.h"
#include "Ring.h"
#include "Triangle.h"
#include "Box.h"
#include "Cylinder.h"
#include "HeightField.h"
#include "PhongMaterial.h"
#include "MetalMaterial.h"
#include "DielectricMaterial.h"
#include "GriddedGroup.h"
#include "PhongMarbleMaterial.h"
#include "SmoothTriangle.h"
#include "WoodMaterial.h"
#include "TileMaterial.h"
#include "Filter.h"
#include "Sampler.h"
#include "PhongImageMaterial.h"
#include "SpherePolar.h"
#include "ObjReader.h"
#include "AreaLight.h"
#include "Instance.h"
#include "PrimitiveInstance.h"
#include "PhongBumpMaterial.h"
#include "ThinLensCamera.h"

#include <fstream>
#include <iostream>
#include <vector>
using std::cerr;
using std::ifstream;
using std::vector;

using namespace std;

Scene* make_scene()
{
	Material* yellow = new PhongMaterial(Color(0.9, 0.9, 0), 0.6, 0.4, Color(1,1,1), 200);
	Material* blue = new PhongMaterial(Color(0.0, 0.0, 1.0), 0.6, 0.4, Color(1,1,1), 200);
	Material* green = new PhongMaterial(Color(0.0, 1.0, 0.0), 0.6, 0.4, Color(1,1,1), 200);
	Material* red = new PhongMaterial(Color(1.0, 0.0, 0.0), 0.6, 0.4, Color(1,1,1), 200);


	Scene* scene = new Scene();
	scene->setBackground(new GradientBackground(Color(.4, .4, .4), Color(106.0/255.0, 90.0/255.0, 58.0/255.0), Vector(0, 1, 0)));
	Group* world = new Group();

	world->addObject(new Sphere(yellow, Point(2.0, 0.0, 0.0), 1.5));
	world->addObject(new Sphere(green, Point(-3.0, 0.0, 20.0), 1.5));
	world->addObject(new Sphere(red, Point(-2.0, 0.0, -20.0), 1.5));
	world->addObject(new Box(blue, Point(-30.0, -2.0, -30.0), Point(30.0, -1.5, 30.0)));	

	scene->setObject(world);
	
	int subDivisions = 5;
	int support = 3;
	scene->setSampler(new JitteredSampler(subDivisions));
	scene->setFilter(new GaussianFilter(support));
	
	
	scene->setAmbient(Color(.2, .2, .25));
	//scene->addLight(new PointLight(Point(20, 70, 100), Color(.9,.9,.9)));
	scene->addLight(new AreaLight(Point(20, 70, 100), Point(25, 70, 100), Point(20, 77, 100), Color(.9,.9,.9)));	
	scene->setCamera(new ThinLensCamera(Point(-1,1,30),
									   Point(-1.0, 0.0, 0.0),
									   Vector(0, 1, 0),
									   12, 11.0, 30.0));

//	scene->setCamera(new PinholeCamera(Point(-1,1,30),
//									   Point(-1.0, 0.0, 0.0),
//									   Vector(0, 1, 0),
//									   12));
	scene->setMaxRayDepth(25);
	scene->setMinAttenuation(.01);
	
	return scene;
}

